unit SameGame;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, SameGameStructure, QMenus, SameGameAbout;

type
  TSameGameForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    HighScore: TMenuItem;
    N2: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    XEdit: TEdit;
    YEdit: TEdit;
    ColEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    SizeEdit: TEdit;
    ExitGame: TMenuItem;
    ScoreEdit: TEdit;
    NextScoreEdit: TEdit;
    ScoreLbl: TLabel;
    NextScoreLbl: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure ExitGameClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure HighScoreClick(Sender: TObject);
  private
    Sphere: TSphere;
  public
    NumColStart: integer;
    procedure GameWon(Sender:TObject);
    procedure GameOver(Sender:TObject);
    procedure BricksRemoved(Sender: TObject; NumOfBricks: integer);
    procedure BricksSearched(Sender: TObject; NumOfBricks: integer);
    procedure ChangeState();
  end;
  THighScore= record
                Score: integer;
                Name: string[50];
              end;
var
  SameGameForm: TSameGameForm;
  SameGameAboutBox: TAboutBox;

implementation

{$R *.xfm}
{$I+}



procedure TSameGameForm.GameWon(Sender:TObject);
var
  x,y,Col:integer;
begin
  Col:=(StrToInt(ColEdit.Text)+1); //Put one more colour in sphere
  ColEdit.Text:=IntToStr(Col);
  x:=StrToInt(XEdit.Text);
  y:=StrToInt(YEdit.Text);
  Sphere.CreateBricks(x,y,Col); //Create bricks
end;

procedure TSameGameForm.GameOver(Sender:TObject);
var
  HighFile: file of THighScore;
  FileName: string;
  HighScore: THighScore;
begin
  ChangeState(); //Show correct user interface
  Sphere.Visible:=false;
  Width:=300;
  Height:=250;
  if Sender<>Self then //if GameOver thrown by sphere check highscore files
  begin
  FileName:='H'+IntToStr(Sphere.XMax)+'_'+IntToStr(Sphere.YMax)+'_'+IntToStr(NumColStart);
  AssignFile(HighFile,FileName);
  try
    ReSet(HighFile);
    Read(HighFile,HighScore);
    CloseFile(HighFile);
    if HighScore.Score<StrToInt(ScoreEdit.Text) then //This is a highscore
    begin
      HighScore.Name:=InputBox('Congratulation, '+ScoreEdit.Text+' is a highscore!','Please enter your name','Anonymous');
      HighScore.Score:=StrToInt(ScoreEdit.Text);
      AssignFile(HighFile,FileName);
      try
        ReWrite(HighFile);
        Write(HighFile,HighScore);
      finally
        CloseFile(HighFile);
      end;
    end
  except
    on EInOutError do //This is a new size played, hence a highscore
    begin
      HighScore.Name:=InputBox('Congratulation, '+ScoreEdit.Text+' is a highscore!','Please enter your name','Anonymous');
      HighScore.Score:=StrToInt(ScoreEdit.Text);
      AssignFile(HighFile,FileName);
      ReWrite(HighFile);
      Write(HighFile,HighScore);
      CloseFile(HighFile);
    end;
  end;
  end;
  ColEdit.Text:=IntToStr(NumColStart);
end;

procedure TSameGameForm.BricksRemoved(Sender: TObject; NumOfBricks: integer);
begin
  ScoreEdit.Text:=IntToStr(StrToInt(ScoreEdit.Text)+StrToInt(NextScoreEdit.Text)); //New score
end;

procedure TSameGameForm.BricksSearched(Sender: TObject; NumOfBricks: integer);
begin
  //Potential score if bricks are clicked
  NextScoreEdit.Text:=IntToStr(NumOfBricks*2*(StrToInt(ColEdit.Text))*(StrToInt(ColEdit.Text)));
end;

procedure TSameGameForm.FormCreate(Sender: TObject);
begin
  Sphere:=TSphere.Create(Self);
  Sphere.Parent:=Self;
  Sphere.onGameWon:=GameWon;
  Sphere.onGameOver:=GameOver;
  Sphere.onBricksRemoved:=BricksRemoved;
  Sphere.onBricksSearched:=BricksSearched;
  SameGameAboutBox:=TAboutBox.Create(Self);
  SameGameAboutBox.Parent:=Self;
end;

procedure TSameGameForm.New1Click(Sender: TObject);
var
  x,y,Col,BrickSize:integer;
begin
  try
    x:=StrToInt(XEdit.Text); //New coordinates
    y:=StrToInt(YEdit.Text);
    Col:=StrToInt(ColEdit.Text);
    if Col<2 then
    begin
      Col:=2;
      ColEdit.Text:='2';
    end;
    if ScoreEdit.Visible=false then //change user inteface to game state
    begin
      ChangeState();
      Sphere.Visible:=true;
    end;
    BrickSize:=StrToInt(SizeEdit.Text);
    Sphere.ChangeBrickSize(BrickSize); //Create a sphere of bricks
    Sphere.CreateBricks(x,y,Col);
    Sphere.Left:=10;
    Sphere.Top:=30;
    width:=trunc((x)*BrickSize)+20; //Set size of form
    if width<300 then width:=300;
    height:=trunc((y+2)*BrickSize)+50;
    if height<250 then height:=250;
    NumColStart:=StrToInt(ColEdit.Text);
    ScoreEdit.Text:='0';
    NextScoreEdit.Text:='0'; //ready to play
  except
    on EConvertError do ShowMessage('Must be integers');
  end;
end;

procedure TSameGameForm.ChangeState();
begin
  XEdit.Visible:=not(XEdit.Visible);
  YEdit.Visible:=not(YEdit.Visible);
  ColEdit.Visible:=not(ColEdit.Visible);
  SizeEdit.Visible:=not(SizeEdit.Visible);
  Label1.Visible:=not(Label1.Visible);
  Label2.Visible:=not(Label2.Visible);
  Label3.Visible:=not(Label3.Visible);
  Label4.Visible:=not(Label4.Visible);
  ScoreLbl.Visible:=not(ScoreLbl.Visible);
  NextScoreLbl.Visible:=not(NextScoreLbl.Visible);
  ScoreEdit.Visible:=not(ScoreEdit.Visible);
  NextScoreEdit.Visible:=not(NextScoreEdit.Visible);
end;

procedure TSameGameForm.ExitGameClick(Sender: TObject);
begin
  if Label1.visible=false then GameOver(Self); //Stop the game without highscoring
end;

procedure TSameGameForm.About1Click(Sender: TObject);
begin
  SameGameAboutBox:=TAboutBox.Create(Self);
  SameGameAboutBox.Parent:=Self;
  SameGameAboutBox.ShowModal;
end;

procedure TSameGameForm.Exit1Click(Sender: TObject);
begin
  close;
end;

procedure TSameGameForm.Open1Click(Sender: TObject);
var
  GameFile: file of integer;
  Score: integer;
  BrickSize: integer;
  NumCol:integer;
begin
  if OpenDialog1.Execute then
  begin
    AssignFile(GameFile,OpenDialog1.FileName);
    try
      ReSet(GameFile);  //Read score and such
      Read(GameFile,Score);
      Read(GameFile,NumColStart);
      Read(GameFile,NumCol);
      ScoreEdit.Text:=IntToStr(Score);
      ColEdit.Text:=IntToStr(NumCol);
    finally
      CloseFile(GameFile);
    end;
    Sphere.OpenBricks(OpenDialog1.FileName+'1'); //Read bricks
    XEdit.Text:=IntToStr(Sphere.XMax);
    YEdit.Text:=IntToStr(Sphere.YMax);
    if ScoreEdit.Visible=false then //Change user interface
    begin
      ChangeState();
      Sphere.Visible:=true;
    end;
    BrickSize:=StrToInt(SizeEdit.Text);
    Sphere.Left:=10;
    Sphere.Top:=30;
    width:=trunc((Sphere.XMax)*BrickSize)+20;
    if width<300 then width:=300;
    height:=trunc((Sphere.YMax+2)*BrickSize)+50;
    if height<250 then height:=250;
  end;
end;

procedure TSameGameForm.SaveAs1Click(Sender: TObject);
var
  GameFile: file of integer;
  Score: integer;
  NumCol: integer;
begin
  if SaveDialog1.Execute then
  begin
    AssignFile(GameFile,SaveDialog1.FileName);
    ReWrite(GameFile);
    try
      Score:=StrToInt(ScoreEdit.text);
      write(GameFile,Score); //Save score and such
      write(GameFile,NumColStart);
      NumCol:=StrToInt(ColEdit.Text);
      write(GameFile,NumCol);
    finally
      CloseFile(GameFile);
    end;
    Sphere.SaveBricks(SaveDialog1.FileName+'1'); //Save bricks
  end;
end;

procedure TSameGameForm.HighScoreClick(Sender: TObject);
var
  HighFile: file of THighScore;
  FileName: string;
  HighScore: THighScore;
begin
  FileName:='H'+XEdit.Text+'_'+YEdit.Text+'_'+ColEdit.Text; //name of correct highscore file
  AssignFile(HighFile,FileName);
  try
    ReSet(HighFile);
    Read(HighFile,HighScore); //Find highscore
    CloseFile(HighFile);
    ShowMessage('Best score is '+IntToStr(HighScore.Score)+', made by '+HighScore.Name);
  except
    on EInOutError do
    begin
      ShowMessage('Nobody has played this grid yet...'); //There is no highscore available
    end;
  end;
end;

end.
