unit SameGameStructure;

interface
uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, QExtCtrls;

type
  //Eventhandler type for scoring.
  TonBricks = procedure(Sender: TObject; NumOfBricks: integer) of object;

  //The playing bricks
  TBricks  =  class(TShape)
              private
                x,y:integer; //Coordinates in the sphere
              end;

  //Used when saving and loading
  TBricksRecord = record
                    x,y: integer;
                    Colour: TColor;
                  end;

  //The playing sphere
  TSphere  =  class (TPanel)
              private
                OldObj: TBricks; //Used to check if the object the mouse is over is the same
                Obj: TBricks; //Hold the different bricks
                DList: TList; //y-axis of the sphere
                Bricks: Tlist; //x-axis of the sphere
                Trash: TList;  //Holds trashable bricks
                NumOfColours: Integer; //Num of colours in the sphere played
                BricksCreated: Boolean; //true when playable bricks exists
                FBrickSize: Integer;
                FonGameOver: TNotifyEvent;
                FonGameWon: TNotifyEvent;
                FonBricksRemoved: TonBricks;
                FonBricksSearched: TonBricks;
                procedure Clicked(Sender: TObject); //Event when bricks are clicked
                procedure ColourSearch(Col: TColor; x,y: Integer; var S: TList); //Find adjecent bricks of same color
                function MovePossible():boolean; //if false GameOver
                procedure RemoveBricks(); //Removes all bricks from sphere
                procedure RefreshBricks(); //Refreshes sphere e.g removes empty y-axis
                procedure RefreshBricksColomn(DList:TList); //Refreshes y-axis e.g bricks falls down
                function GetBrick(x,y: Integer):TBricks; //Returns brick at coordinate
                function GetYMaxColomn(x:Integer):Integer; //Returns num of active x-axis
                function BrickExist(x,y: Integer):Boolean; //if brick at coor. exits returns true
                procedure MMove(Sender: TObject;Shift: TShiftState; X, Y: Integer); //Event for mouse moving over brick
                procedure MEnter(Sender:TObject); //Event for mouse entering brick
                procedure MLeave(Sender:TObject); //Event for mouse leaving brick
              public
                XMax,YMax:Integer; //Max coordinates of sphere
                procedure ChangeBrickSize(Size: Integer); //Change the size of all bricks
                procedure CreateBricks(x,y,Col:Integer); //Creates all bricks
                procedure SaveBricks(FileName: String);  //Saves all bricks
                procedure OpenBricks(FileName: String);  //Load all bricks
                constructor Create(Owner:TComponent);override;
                destructor Destroy; override;
              protected
                procedure Paint();override; //Gamewon, GameOver thrown here. Cleans up trash
              published
                property BrickSize: Integer read FBrickSize write ChangeBrickSize default 20;
                property onGameOver: TNotifyEvent read FonGameOver write FonGameOver;
                property onGameWon: TNotifyEvent read FonGameWon write FonGameWon;
                property onBricksRemoved: TonBricks read FonBricksRemoved write FonBricksRemoved;
                property onBricksSearched: TonBricks read FonBricksSearched write FonBricksSearched;
              end;
  ESphereError = class(exception)
              end;

implementation



procedure TSphere.MMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if OldObj<>Sender then //Mouse over new brick
  begin
    MLeave(OldObj);
    MEnter(Sender);
    if OldObj.Visible=false then OldObj.Free; //free memory if OldObj is removed
    OldObj:=(Sender as TBricks); //OldObj holds the new Brick
  end;
end;

procedure TSphere.MEnter(Sender:TObject);
var
  S: TList; //To hold all adjecent bricks with same colour
  Col: TColor;
begin
  S:=TList.Create;
  if (Sender is TBricks) then with (Sender as TBricks) do
  begin
    Col:=Brush.Color;
    ColourSearch(Col,x,y,S); //Finds all the objects
    //Bricks have been seardch so if the game can proceed onBrickSearched is thrown
    if (assigned(onBricksSearched)) and MovePossible() then onBricksSearched(Self, S.Count);
    while S.Count>0 do  //change the boarder colour of all the bricks adjecent
    begin
      Obj:=S.Last;
      S.Delete(S.Count-1);
      Obj.Pen.Color:=Obj.Brush.Color;
    end;
  end;
  S.Free;
end;

procedure TSphere.MLeave(Sender: TObject);
var
  S: TList;  //To hold all adjecent bricks with same colour
  Col: TColor;
begin
  S:=TList.Create;
  if (Sender is TBricks) then with (Sender as TBricks) do
  begin
    Col:=Brush.Color;
    ColourSearch(Col,x,y,S);
    while S.Count>0 do //change boarder colour back to black
    begin
      Obj:=S.Last;
      S.Delete(S.Count-1);
      Obj.Pen.Color:=clBlack;
    end;
  end;
  S.Free;
end;
 
constructor TSphere.Create(Owner:TComponent);
begin
  inherited Create(Owner);
  visible:=true;
  Bricks:=TList.Create;
  DList:=TList.Create;
  Trash:=TList.Create;
  BricksCreated:=false;
  BrickSize:=20;
  randomize;
end;

destructor TSphere.Destroy;
begin
  while Trash.Count>0 do
  begin
    Obj:=Trash.Last;
    Trash.Delete(Trash.Count-1);
    Obj.Free;
  end;
  RemoveBricks();
  Bricks.Free;
  Trash.Free;
  inherited destroy;
end;



procedure TSphere.CreateBricks(x,y,Col:Integer);
var i,j:integer;
begin
  while Trash.Count>0 do
  begin
    Obj:=Trash.Last;
    Trash.Delete(Trash.Count-1);
    Obj.Free;
  end;
  BricksCreated:=false; //Not ready to play
  OldObj:=TBricks.Create(Self);
  OldObj.Parent:=Self;
  OldObj.Visible:=false;
  RemoveBricks(); //Now any pre-played object has been cleaned
  NumOfColours:=Col;
  XMax:=x;
  YMax:=y;
  width:=(XMax)*BrickSize;
  Height:=(YMax)*BrickSize; //The sphere is ready
  for i:=1 to XMax do //All new bricks are created
  begin
    DList:=TList.Create;
    for j:=1 to YMax do
    begin
      Obj:=TBricks.Create(Self);
      Obj.Parent:=Self;
      Obj.Width:=BrickSize;
      Obj.Height:=BrickSize;
      Obj.OnClick:=Clicked;
      Obj.OnMouseMove:=MMove;
      case Random(NumOfColours) of
        0: Obj.Brush.Color:=clBlue;
        1: Obj.Brush.Color:=clGreen;
        2: Obj.Brush.Color:=clRed;
        3: Obj.Brush.Color:=clYellow;
        4: Obj.Brush.Color:=clBlack;
        5: Obj.Brush.Color:=clTeal;
        6: Obj.Brush.Color:=clOlive;
        7: Obj.Brush.Color:=clSilver;
      else
        Obj.Brush.Color:=clMaroon;
      end;
      Obj.Visible:=true;
      DList.Add(Obj);
    end;
    Bricks.Add(DList);
  end;
  RefreshBricks(); //Refresh the bricks (give them their coodinates)
  BricksCreated:=true; //Ready to play
end;

procedure TSphere.RemoveBricks();
begin
  while Bricks.Count>0 do //Frees all objects in the sphere
  begin
    Dlist:=Bricks.Last;
    Bricks.Delete(Bricks.Count-1);
    while Dlist.Count>0 do
    begin
      obj:=DList.Last;
      DList.Delete(DList.Count-1);
      Obj.Free;
    end;
    DList.Free;
  end;
end;

procedure TSphere.Paint();
begin
  inherited;
  //GameOver
  if not(MovePossible()) and (bricks.Count>0) and assigned(onGameOver) then
  begin
    BricksCreated:=false;
    onGameOver(Self);
  end;
  //GameWon
  if assigned(onGameWon) and (Bricks.Count=0) and BricksCreated then
  begin
    BricksCreated:=false;
    onGameWon(Self);
  end;
  //Cleans up the trash
  while Trash.Count>0 do
  begin
    Obj:=Trash.Last;
    Trash.Delete(Trash.Count-1);
    Obj.Free;
  end;
end;

procedure TSphere.ChangeBrickSize(Size: Integer);
var
  i,j:Integer;
begin
  FBrickSize:=Size;
  width:=(XMax)*BrickSize; //Size of sphere
  Height:=(YMax)*BrickSize;
  i:=0;
  while i<(Bricks.Count) do  //all sizes of bricks are changed
  begin
    DList:=Bricks[i];
    j:=0;
    while j<(DList.Count) do
    begin
      Obj:=DList[j];
      Obj.Width:=BrickSize;
      Obj.Height:=BrickSize;
      inc(j);
    end;
    inc(i);
  end;
  RefreshBricks(); //New visual coodinates given
end;

procedure TSphere.Clicked(Sender: TObject);
var
  S: TList;
begin
  S:=TList.Create;
  if (Sender is TBricks) then
  with (Sender as TBricks) do
  begin
    Obj:=GetBrick(x,y);
    S.Add(Obj);
    ColourSearch(Obj.Brush.Color,x,y,S);
    //Some object are removed, throw an onBricksRemoved event
    if S.Count>0 then if assigned(onBricksRemoved) then onBricksRemoved(Self, S.Count);
    while S.Count>0 do
    begin
      obj:=S.Last; //Gets an brick to remove
      S.Delete(S.Count-1); //delete it from the list
      if Obj=OldObj then   //Then a dummy obj is created to avoid checking a destroyed object
      begin
        OldObj:=TBricks.Create(Self);
        OldObj.Parent:=Self;
        OldObj.Visible:=False;
      end;
      Trash.Add(Obj); //add obj to the trash
      Obj.Visible:=false;
      DList:=Bricks[Obj.x];
      DList.Delete(Obj.y);
      if DList.Count=0 then //No more bricks in this x-axis
      begin
        Bricks.Remove(DList); //Removes it from the sphere
        DList.Free; //free the mem
        RefreshBricks(); //find new coordinates
      end
      else RefreshBricksColomn(DList); //Find new coordintes i x-axis
    end;
    S.Free;
  end;
end;


procedure TSphere.ColourSearch(Col: TColor; x,y: Integer; var S: TList);
begin
  if BrickExist(x-1,y) then //Is there a brick to the left
  begin
    obj:=GetBrick(x-1,y); //Get the brick to the left
    //Are they of the same colour, and hasn't it been checked before?
    if (Col=obj.Brush.Color) and (S.IndexOf(obj)=-1) then
    begin
      S.Add(obj); //Add it to the list
      ColourSearch(Col,x-1,y,S); //Do a colour search on the brick to the list
    end;
  end;
  if BrickExist(x+1,y) then //Do the same with the brick to the right
  begin
    obj:=GetBrick(x+1,y);
    if (Col=obj.Brush.Color) and (S.IndexOf(obj)=-1) then
    begin
      S.Add(obj);
      ColourSearch(Col,x+1,y,S);
    end;
  end;
  if BrickExist(x,y-1) then //Brick under
  begin
    obj:=GetBrick(x,y-1);
    if (Col=obj.Brush.Color) and (S.IndexOf(obj)=-1) then
    begin
      S.Add(obj);
      ColourSearch(Col,x,y-1,S);
    end;
  end;
  if BrickExist(x,y+1) then //Brick over
  begin
    obj:=GetBrick(x,y+1);
    if (Col=obj.Brush.Color) and (S.IndexOf(obj)=-1) then
    begin
      S.Add(obj);
      ColourSearch(Col,x,y+1,S);
    end;
  end;
  if S.Count=1 then S.Delete(0); //If no bricks found, remove the clicked brick from the list
end;


procedure TSphere.RefreshBricks();
var
  NumOfCol: Integer;
  i,j: Integer;
begin
  NumOfCol:=Bricks.Count;
  if NumOfCol>0 then
  begin
    i:=0;
    while (i<NumOfCol) do //Go through all x-axises
    begin
      DList:=Bricks[i];
      DList.Pack;
      for j:=0 to (DList.Count-1) do //Go through all y-axis bricks
      begin
        obj:=DList[j];
        obj.x:=i;
        obj.y:=j;
        obj.Left:=BrickSize*i; //Give new visual x coordinate
        obj.Top:=(YMax-j-1)*BrickSize; //Give new visual y coordinate
      end;
      inc(i);
    end;
  end;
end;

procedure TSphere.RefreshBricksColomn(DList:TList);
var
  j,Max: Integer;
begin
  Max:=DList.Count;
  if Max>0 then
  begin
    for j:=0 to (Max-1) do  //go though all bricks in this x-axis
    begin
      Obj:=DList[j];
      Obj.y:=j;
      Obj.Top:=(YMax-j-1)*BrickSize; //Give new visual y coordinate
    end;
  end
  else
  begin
    Bricks.Pack; //Remove empty x-axis
    RefreshBricks();
  end;
end;

function TSphere.GetBrick(x,y: Integer):TBricks;
begin
  if DList=nil then DList:=TList.Create;
  if BrickExist(x,y) then //Does brick exit
  begin
    DList:=Bricks[x];
    Result:=DList[y]; //this is the brick wanted
  end
  else raise ESphereError.Create('Brick does not exist'); //
end;

function TSphere.GetYMaxColomn(x:Integer):Integer;
begin
  DList:=Bricks[x];
  Result:=DList.Count-1; //The y coordinate of brick at top of x-axis
end;

function TSphere.BrickExist(x,y: Integer):Boolean;
begin
  if (x<0) or (x>Bricks.Count-1) then Result:=False //out of x range
  else
  begin
    if (y<0) or (y>GetYMaxColomn(x)) then Result:=false //out of y range
    else Result:=true; //Brick exits
  end;
end;

function TSphere.MovePossible():boolean;
var
  i,j: Integer;
  S: TList;
begin
  S:=TList.Create;
  Result:=false;
  i:=Bricks.Count;
  while (i>0) and (Result=false) do //Search all bricks or until some bricks found
  begin
    dec(i);
    DList:=Bricks[i];
    j:=DList.Count;
    while (j>0) and (Result=false) do
    begin
      dec(j);
      Obj:=DList[j];
      ColourSearch(Obj.Brush.Color,i,j,S);
      if S.Count>0 then Result:=true; //At least two bricks of of same col are adjecent
    end;
  end;
  S.Free;
end;


procedure TSphere.SaveBricks(FileName: String);
var
  BrickFile: File of TBricksRecord;
  SBrick: TBricksRecord;
  i,j: integer;
begin
  AssignFile(BrickFile,FileName);
  try
    ReWrite(BrickFile);
    for i:=0 to (Bricks.Count-1) do //Save info of all bricks
    begin
      DList:=Bricks[i];
      for j:=0 to (DList.Count-1) do
      begin
        Obj:=DList[j];
        SBrick.x:=Obj.x;
        SBrick.y:=Obj.y;
        SBrick.Colour:=Obj.Brush.Color;
        write(BrickFile,SBrick);
      end;
    end;
  finally
    CloseFile(BrickFile);
  end;
end;

procedure TSphere.OpenBricks(FileName: String);
var
  BrickFile: File of TBricksRecord;
  SBrick: TBricksRecord;
  i,m,n: integer;
  S:TList;
begin
  BricksCreated:=false; //Not ready to play
  S:=TList.Create;
  OldObj:=TBricks.Create(Self); //Give OldObj a dummy to avoid rumtime errors
  OldObj.Parent:=Self;
  OldObj.Visible:=false;
  RemoveBricks();
  XMax:=1;
  YMax:=1;
  AssignFile(BrickFile,FileName);
  try
    Reset(BrickFile);
    while not(Eof(BrickFile)) do //Read until end of file, and initialize bricks
    begin
      Read(BrickFile,SBrick);
      Obj:=TBricks.Create(Self);
      Obj.Parent:=Self;
      Obj.x:=SBrick.x;
      Obj.y:=SBrick.y;
      Obj.Brush.Color:=SBrick.Colour;
      Obj.Width:=BrickSize;
      Obj.Height:=BrickSize;
      Obj.OnClick:=Clicked;
      Obj.OnMouseMove:=MMove;
      S.Add(Obj);
      if Obj.x>(XMax-1) then XMax:=Obj.x+1;
      if Obj.y>(YMax-1) then YMax:=Obj.y+1;
    end;
  Finally
    CloseFile(BrickFile);
  end;
  width:=(XMax)*BrickSize; //Set sphere size
  Height:=(YMax)*BrickSize;
  for i:=1 to XMax do  //Create all x-axis needed
  begin
    DList:=TList.Create;
    Bricks.Add(DList);
  end;
  for i:=0 to (S.Count-1) do //Put in all bricks at their position
  begin
    Obj:=S[i];
    m:=Obj.x;
    n:=Obj.y;
    DList:=Bricks[m];
    DList.Insert(n,Obj);
  end;
  RefreshBricks();
  BricksCreated:=true; //ready to play
end;

end.
